#include "skipper.h"
#include "data.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

static int dig(const char *what, const unsigned char *where)
{
  int i, j;
  j = 0;
  for (i = 0; i < 65536-0xb4; i++) {
    if (what[j] == where[i]) {
      j++;
      if (what[j] == 0) return i-j+1;
    } else {
      i -= j;
      j = 0;
    }
  }
  return -1;
}

void init_skipper(skipper *s, const char *file, int buf_size)
{
  FILE *f;
  static unsigned char an_buf[65536];
  int pos;

  s->pre = s->post = 0;

  memset(an_buf, 0, 65536);
  f = fopen(file, "rb"); if (!f) goto safefail;
  if (fread(an_buf, 65536, 1, f)) /* shut up gcc */;
  fclose(f); f = NULL;
  pos = dig("Xing", an_buf);
  if (pos == -1) pos = dig("Info", an_buf);
  if (pos == -1) goto safefail;
  pos -= 36;
  if (an_buf[pos] != 0xff || an_buf[pos+1] != 0xfb ||
      an_buf[pos+0x9c] != 'L' ||
      an_buf[pos+0x9d] != 'A' ||
      an_buf[pos+0x9e] != 'M' ||
      an_buf[pos+0x9f] != 'E') goto safefail;

  s->pre = (an_buf[pos+0xb1]<<4) | ((an_buf[pos+0xb2]>>4)&15);
  s->post = ((an_buf[pos+0xb2]&15)<<8) | an_buf[pos+0xb3];

printf("MP3 pre %d post %d\n", s->pre, s->post);

safefail:
  s->pre *= 4;
  s->post *= 4;
  s->size = s->post + buf_size;
  s->data = malloc(s->size); if (!s->data) OOM;
  s->next = 0;
}

void free_skipper(skipper *s)
{
  free(s->data);
}

void feed_skipper(skipper *s, const unsigned char *data, int len, obuf *b)
{
  /* speed this up, go by blocks */
  while (s->pre && len) { s->pre-=4; len-=4; data+=4; }
  while (len) {
    if (s->next == s->size) {
      obuf_put(b, s->data, s->size-s->post);
      memmove(s->data, s->data+s->size-s->post, s->post);
      s->next = s->post;
    }
    s->data[s->next] = *data;
    data++;
    s->next++;
    len--;
  }
}

void flush_skipper(skipper *s, obuf *b)
{
  if (s->next - s->post < 0) abort();
  if (s->next - s->post == 0) return;
  obuf_put(b, s->data, s->next-s->post);
}
